module ADS54J40_CPLD
(

// Inputs from FT245RL
input 	FTDI_SCLK,
input 	FTDI_SDIO,
output 	FTDI_SDO,
input 	FTDI_SEN_ADC,
input 	FTDI_SEN_LMK,
input 	FTDI_SEN_LMH, //Uncomment to use the SDIO_READ line of the FTDI device
output 	FTDI_LMK_SDIO,
input 	FTDI_DIR_CONTROL,

// SPI Inputs from FMC Connector
input 	FMC_SCLK,
input 	FMC_SDIO,
output 	FMC_SDO,
input 	FMC_SEN_ADC,
input 	FMC_SEN_LMK,
input 	FMC_SEN_LMH, //Uncomment to use the SDIO_READ line of the FMC port
output 	FMC_LMK_SDIO,
input 	FMC_DIR_CONTROL,

// FMC or FTDI SPI Selector
input		FMC_OR_FTDI_SELECTOR,

// ADC SPI control lines
output	ADC_SCLK,
output	ADC_SDENB,
output	ADC_SDIO,
input		ADC_SDO,

// LMK SPI control lines
output 	LMK_SCK,
output 	LMK_CS,
inout 	LMK_SDIO,
input 	LMK_SDO,

// LMH SPI control lines
output	AMP_SCLK,
output	AMP_SDIO,
output	AMP_CS,
input		AMP_SDO

//input SYSREF_BUTTON,
//output LMK_CPLD_SYNC

);

assign ADC_SDIO = SDIO_ROUTE;
assign LMK_SDIO = (DIR_CONTROL_ROUTE) ? SDIO_ROUTE : 1'bz;
assign AMP_SDIO = SDIO_ROUTE;

// Control routing of SPI signals to/from the FMC connector and FTDI chip
// If the control pin is high, take signals from FMC
// If the control pin is low, take signals from FTDI
reg SCLK_ROUTE;
reg SDIO_ROUTE;
reg SEN_ADC_ROUTE;
reg SEN_LMK_ROUTE;
reg SEN_LMH_ROUTE;
reg DIR_CONTROL_ROUTE;
always @ (*) begin
	if(FMC_OR_FTDI_SELECTOR) begin
		SCLK_ROUTE <= FMC_SCLK;
		SDIO_ROUTE <= FMC_SDIO;
		SEN_ADC_ROUTE <= FMC_SEN_ADC;
		SEN_LMK_ROUTE <= FMC_SEN_LMK;
		SEN_LMH_ROUTE <= FMC_SEN_LMH;
		DIR_CONTROL_ROUTE <= FMC_DIR_CONTROL;
	end else begin
		SCLK_ROUTE <= FTDI_SCLK;
		SDIO_ROUTE <= FTDI_SDIO;
		SEN_ADC_ROUTE <= FTDI_SEN_ADC;
		SEN_LMK_ROUTE <= FTDI_SEN_LMK;
		SEN_LMH_ROUTE <= FTDI_SEN_LMH;
		DIR_CONTROL_ROUTE <= FTDI_DIR_CONTROL;
	end
end

// Route SCLK straight from FMC/FTDI to ADC/LMK
assign ADC_SCLK = SCLK_ROUTE;
assign LMK_SCK = SCLK_ROUTE;
assign AMP_SCLK = SCLK_ROUTE;

// Route SEN straight from FMC/FTDI to ADC/LMK
assign ADC_SDENB = SEN_ADC_ROUTE;
assign LMK_CS = SEN_LMK_ROUTE;
assign AMP_CS = SEN_LMH_ROUTE;

/* Disable using SDIO of the FTDI chip as readback.
	Uncomment to allow SDIO to be readback in 3-wire mode.
	Also, change the FTDI and FMC SDIO pins to inout.

// Control direction for 3-wire and 4-wire interface
// 1'b1 = write to device, 1'b0 = read from device
reg ADC_LMK_SELECTION;

// If the LMK SEN is active (low) and the ADC SEN is high,
// then readback from LMK, otherwise readback from ADC
always @ (*) begin
	if(!SEN_LMK_ROUTE && SEN_ADC_ROUTE) begin
		ADC_LMK_SELECTION <= LMK_SDIO;
	end else begin
		ADC_LMK_SELECTION <= ADC_SDIO;
	end
end

assign FTDI_SDIO = (!DIR_CONTROL_ROUTE && !FMC_OR_FTDI_SELECTOR) ? ADC_LMK_SELECTION : 1'bz;
assign FMC_SDIO  = (!DIR_CONTROL_ROUTE && FMC_OR_FTDI_SELECTOR)  ? ADC_LMK_SELECTION : 1'bz;

*/

// Send ADC and LMK SDIO to the individual readback pins
//assign ADC_SDIO = (DIR_CONTROL_ROUTE) ? SDIO_ROUTE : 1'bz;
//assign LMK_SDIO = (DIR_CONTROL_ROUTE) ? SDIO_ROUTE : 1'bz;
//assign FTDI_ADC_SDIO = (!FMC_OR_FTDI_SELECTOR) ? ADC_SDIO : 1'bz; //Uncomment to use the SDIO_READ line of the FTDI device
assign FTDI_LMK_SDIO = (!FMC_OR_FTDI_SELECTOR) ? LMK_SDIO : 1'bz;
//assign FMC_ADC_SDIO = (FMC_OR_FTDI_SELECTOR) ? ADC_SDIO : 1'bz;   //Uncomment to use the SDIO_READ line of the FMC port
assign FMC_LMK_SDIO = (FMC_OR_FTDI_SELECTOR) ? LMK_SDIO : 1'bz;

// Control routing of LMK and ADC SDO to FTDI or FMC
reg SDO_ROUTE;
always @ (*) begin
	if(!SEN_ADC_ROUTE) begin
		SDO_ROUTE <= ADC_SDO;
	end else if(!SEN_LMH_ROUTE) begin
		SDO_ROUTE <= AMP_SDO;
	end else begin
		SDO_ROUTE <= LMK_SDO;
	end
end

assign FTDI_SDO = (!FMC_OR_FTDI_SELECTOR) ? SDO_ROUTE : 1'bz;
assign FMC_SDO = (FMC_OR_FTDI_SELECTOR) ? SDO_ROUTE : 1'bz;
	 
endmodule 